<?php /* _\|/_              vim: set et ts=4 sw=4 sts=4 nowrap fdm=marker:
         (o o)                         
+-----oOO-{_}-OOo--------------------------------------------------------+
| ConPresso - Foolproof Content Management                               |
+------------------------------------------------------------------------+
| Copyright (c) Bartels Schoene GmbH & Co. KG, 48149 Muenster, Germany   |
+------------------------------------------------------------------------+
| This source file is subject to ConPresso license version 2002-11-16    |
| that is bundled with this package in the files LICENSE_EN.TXT or       |
| LICENSE_DE.TXT, and is available through the world-wide-web at         |
|     http://www.conpresso.de/license/LICENSE_EN-2002-11-16.TXT or       |
|     http://www.conpresso.de/license/LICENSE_DE-2002-11-16.TXT          |
|                                                                        |
| If you did not receive a copy of the ConPresso license and are unable  |
| to obtain it through the world-wide-web, please send a note to         |
| <license@conpresso.de> so we can e-mail you a copy immediately.        |
+------------------------------------------------------------------------+
| Authors: ConPresso development and support team <support@conpresso.de> |
+------------------------------------------------------------------------+
  $Id$ */
if (!defined('CPO')) exit;

function printManualTOC($manual, $level = 0)
{
    $toc = '';

    $dlstart = '<dl>'; $dlend = '';
    foreach ($manual as $k=>$v) {
        if (is_array($v)) {
            $toc .= '<dd>';
            $toc .= printManualTOC($v, $level+1); // recursion
            $toc .= '</dd>';

            continue;
        }

        if ($level!=0)
            $toc .= $dlstart; $dlstart = ''; $dlend = '</dl>';

        $linkTo = BASE_REL.'_manual/'.$GLOBALS['cpoSystem']['language'].'/'.$v;

        if (substr($v, 0, 5)=='http:') {
            $linkTo = $v;
            if (isset($_SERVER['HTTPS'])) $linkTo = str_replace('http:', 'https:', $linkTo);
        }
        

        if (substr($v, 0, 1)=='/') $linkTo = BASE_REL.substr($v, 1);

        $toc .= '<dt>';
        $toc .= '<span class="section">';
        $toc .= '<a target="manual_content" href="'.htmlspecialchars($linkTo, ENT_COMPAT, 'ISO-8859-15').'">'.htmlspecialchars($k, ENT_COMPAT, 'ISO-8859-15').'</a>';
        $toc .= '</span>';
        $toc .= '</dt>';
    }
    $toc .= $dlend;

    return $toc;
}

function doActions($action)
{
    if (is_array($action)) { 
        foreach($action as $v) doActions($v); 
        return; 
    }

    if (strncmp($action, 'function:', 9)==0 && 
        preg_match('/^function:\s*(\S+)\s*(.*)\s*$/s', $action, $match) &&
        function_exists($match[1])) { 

        return $match[1]($match[2]); 
    }

    if (strncmp($action, 'include:', 8)==0 && 
        preg_match('/^include:\s*(.+)\s*/s', $action, $match)) {

        $files = preg_split('/[\s]+/', $match[1], -1, PREG_SPLIT_NO_EMPTY);
        foreach($files as $v) {
            $v = urldecode($v); // filenames are encoded, because we need a "space" as divider

            if (file_exists($v) && is_readable($v)) { 
                include($v); 
                return; 
            }

            // also try with prefixed CPO_BASEDIR
            if (file_exists(CPO_BASEDIR.$v) && is_readable(CPO_BASEDIR.$v)) { 
                include(CPO_BASEDIR.$v); 
                return; 
            }
        }
    }

    echo $action;
}

function printIcon($url, $icon, $caption, $active=true)
{
    if ($active) {
        if ($url!='') echo '<a href="'.htmlspecialchars($url, ENT_COMPAT, 'ISO-8859-15').'">';
        echo '<img src="'.$icon.'" width="18" height="21" '
            .'alt="'  .htmlspecialchars($caption, ENT_COMPAT, 'ISO-8859-15').'" '
            .'title="'.htmlspecialchars($caption, ENT_COMPAT, 'ISO-8859-15').'">';
        if ($url!='') echo '</a>';
    } else {
        echo '<img src="'.SKIN_URL.'pic_shade.gif" class="pic" '
            .'style="background: url('.$icon.');" '
            .'alt="'  .htmlspecialchars($caption, ENT_COMPAT, 'ISO-8859-15').'" '
            .'title="'.htmlspecialchars($caption, ENT_COMPAT, 'ISO-8859-15').'">';
    }
}

// necessary to possibly set the session save handlers on each start...
function CPOsession_start() 
{
    if (!$GLOBALS['cpoSystem']['sessionsInDB']) {
        session_start();
    } else {
        // the following prevents unexpected effects when using objects as save handlers
        // required as of PHP 5.0.5
        register_shutdown_function('session_write_close');

        require_once(CPO_BASEDIR.'_include/sessiondb.inc.php');
        session_set_save_handler(
            'sessionDB_open',
            'sessionDB_close',
            'sessionDB_read',
            'sessionDB_write',
            'sessionDB_destroy',
            'sessionDB_gc'
        );
        @session_start();
    }
}

// find the name and type of a key "a name (type)"
function nametype($key, &$name, &$type) {
    preg_match('/^(.*) \((.*)\)$/', $key, $matches);
    if (count($matches)!=3) return false; 
    $name = $matches[1];
    $type = $matches[2];
    return true;
}

function printPageHeader($title, $subtitle, $help='', $nowhitespace=false, $show='', $language = '')
{
    printf('<div class="header"><h1>%s %s</h1>%s</div>',
        $title,
        help($help, $nowhitespace, $show, $language),
        $subtitle
    );

}

function save_article($data) 
{
    global $cpoSystem;

    // check for missing fields
    if (!is_array($data)) return false;
    if (empty($data['autor'])) return false;
    if (empty($data['templates_id'])) return false;
    if (empty($data['titel'])) return false;
    if (empty($data['rubric_id'])) return false;

    if (empty($data['idx'])) $data['idx'] = 'zzzzzzzz';
    if (empty($data['datum'])) $data['datum'] = date('Y-m-d H:i:s');
    if (empty($data['verfallsdatum'])) $data['verfallsdatum'] = '2036-01-01 00:00:00';
    if (empty($data['pub_datum'])) $data['pub_datum'] = date('Y-m-d H:i:s');

    if (!empty($data['nr'])) {
        $_REQUEST['nr'] = $data['nr'];
        require(CPO_BASEDIR.'_include/article_archive.php');
    }

    // insert into DB
    $datum = date('Y-m-d H:i:s' ,time());

/*
// only set release if article was not blocked before... // FIXME why?
if ($_POST['freigabe']!=STATE_BLOCKED) {
    $_POST['freigabe'] = $cpoSystem['release'] ? STATE_WAITING : STATE_RELEASED;
}
*/
    $free = $cpoSystem['release'] ? STATE_WAITING : STATE_RELEASED;
    $free_id = '';
    $free_date = '0000-00-00 00:00:00';
    if ($cpoSystem['release'] && !empty($data['releasedby']) ) {
        $free      = STATE_RELEASED;
        $free_id   = $data['releasedby'];
        $free_date = date('Y-m-d H:i:s', time());

        if (!empty($data['nr'])) {
            // make online article an archived one
            $db = new DB;
            $query = "UPDATE ".CPO_NEWS." "
                    ."SET artikel_archiv=5 "
                    ."WHERE artikel_archiv=1 "
                    ."AND article_nr='".addslashes($data['nr'])."'";
            $db->query($query);
        }
    }

    if (!isset($data['usergroup']) || !is_array($data['usergroup'])) $data['usergroup'] = array();
    $auth = '#'.implode('#', $data['usergroup']).'#';
    $auth = preg_replace('/##+/', '#', $auth);
    if ($auth=='#') $auth = '';

    $db = new DB;

    if (!empty($data['nr'])) {
        $query = "DELETE FROM ".CPO_NEWS." "
                ."WHERE nr='".addslashes($data['nr'])."'";
        $db->query($query); 
    }
    
    initVar($data['nr'],NULL);
    initVar($data['keywords'],'');
    initVar($data['commentary'],'');
    initVar($data['rubric_id'],'');
    initVar($data['templates_id'],'');
    initVar($data['titel'],'');
    initVar($data['datum'],'');
    initVar($data['verfallsdatum'],'');
    initVar($data['pub_datum'],'');
    initVar($data['autor'],'');
    initVar($data['email'],'');
    initVar($data['initial'],'');
    initVar($data['idx'],'');
    initVar($data['changedBy'],'');
    initVar($data['sort_id'],'');

    $query = "INSERT INTO ".CPO_NEWS." "
           ."(nr, artikel_archiv, article_nr, auth, keywords, commentary, rubric_id, templates_id, titel, datum, verfallsdatum, pub_datum, autor, email, initial, idx, freigabe, released_user_id, release_date, last_modify, c_time, changer, sort_id) "
            ."VALUES ("
           ."'".addslashes($data['nr'])."', "           // nr
            ."'0', "                                    // artikel_archiv
           ."'".addslashes($data['nr'])."', "           // article_nr
            ."'".addslashes($auth)."', "                // auth
            ."'".addslashes($data['keywords'])."', "    // keywords
            ."'".addslashes($data['commentary'])."', "  // commentary
            ."'".addslashes($data['rubric_id'])."', "   // rubric_id
            ."'".addslashes($data['templates_id'])."', " // templates_id
            ."'".addslashes($data['titel'])."', "       // titel
            ."'".addslashes($data['datum'])."', "       // datum
            ."'".addslashes($data['verfallsdatum'])."', " // verfallsdatum
            ."'".addslashes($data['pub_datum'])."', "   // pub_datum
            ."'".addslashes($data['autor'])."', "       // autor
            ."'".addslashes($data['email'])."', "       // e-mail  
            ."'".addslashes($data['initial'])."', "     // initial      
            ."'".addslashes($data['idx'])."', "         // idx
            ."'".addslashes($free)."', "                // freigabe
            ."'".addslashes($free_id)."', "             // released_user_id
            ."'".addslashes($free_date)."', "           // release_date
            ."'".addslashes($datum)."', "               // last_modify
            ."'".addslashes($datum)."', "               // c_time
            ."'".addslashes($data['changedBy'])."', "   // changer // SID_login on replace
            ."'".addslashes($data['sort_id'])."') ";    // sort_id
    DEBUG(1, 'Inserting metadata of article.<br /><br />'.$query, __FILE__, __LINE__);
    $db->query($query); 

    $nr = $data['nr'];
    if (empty($nr)) {
        $query = 'SELECT last_insert_id()';
        $db->query($query); 
        $db->next_record();
        $nr = $db->v('last_insert_id()');

        $query = "UPDATE ".CPO_NEWS." SET "
                ."article_nr='".addslashes($nr)."', " // article_nr = nr for newest article version
                ."sort_id='".addslashes($nr)."' "     // use sort_id based on nr (auto-incremented)
                ."WHERE nr='".addslashes($nr)."'";
        $db->query($query); 
    }

    require(CPO_BASEDIR.'_include/insertdb_fields.inc.php');

    return $nr;
}

function printHumanReadable($bytes) {
    $unit = ' Byte';
    if ($bytes != 0) {
        $bytes = abs($bytes);
        if ($bytes < 1024 ) {
            $unit = ' Byte';
        } else if ($bytes < 1024 * 1024) {
            $unit = ' KByte';
            $bytes /= 1024;
        } else if ($bytes < 1024 * 1024 * 1024) {
            $unit = ' MByte';
            $bytes /= 1024 * 1024;
        } else if ($bytes < 1024 * 1024 * 1024 * 1024) {
            $unit = ' GByte';
            $bytes /= 1024 * 1024 * 1024;
        } else {
            $unit = ' TByte';
            $bytes /= 1024 * 1024 * 1024 * 1024;
        }
    }
    return (sprintf("%.1f".$unit, $bytes));
}


function string_next_page($first, $total, $perpage, $found=false, $zusatz_url='', $targetURL='', $tl=array(), $defaultPerPage = NULL) 
/*
next_page($pos, $max_data,$cpoSystem['noOfArticles'],1,"1","release.php");
max_data: anzahl ALLER Datensaetze
$perpage: anzahl ANZUZEIGENDER
*/
{
    $pager = '';
    $pager .= "\n\t\t\t<!-- Seiten-Navigation -->\n\t\t\t";  // MR: Einrckung

    if ($targetURL=='') $targetURL = $_SERVER['PHP_SELF'];

    if (empty($tl['Found:']))        $tl['Found:'] = __('Found:');
    if (empty($tl['Results:']))      $tl['Results:'] = __('Results:');
    if (empty($tl['previous page'])) $tl['previous page'] = __('previous page');
    if (empty($tl['next page']))     $tl['next page'] = __('next page');
    if (empty($tl['all']))           $tl['all'] = __('all');
    if (empty($perpage)) $perpage = $defaultPerPage;
    $all = false;
    if ($perpage=='all') {
        $perpage = $total;
        $all = true;
    }
    
    $pages      = floor(($total+$perpage-1)/$perpage); // no of pages
    $activePage = floor($first/$perpage);              // actual ("active") page
    $last       = $first+$perpage;                     // last element
    $nextSpan   = max(10,ceil($pages/100)*10);
    if ($last>$total) $last = $total;
    
    // no need to display prev, next
    if ($defaultPerPage==NULL && $total<=$perpage) return '';

    $pager .= '<div class="pager">';
    if ($defaultPerPage!=NULL) {
        $pager .= '<form method="post" action="' . htmlspecialchars($targetURL, ENT_COMPAT, 'ISO-8859-15') . '?'.$zusatz_url.'">';
    }
        
    if ($found) {
        $pager .= '<strong>'.htmlspecialchars($tl['Found:'], ENT_COMPAT, 'ISO-8859-15').'</strong> '.(int)$total.' | ';
        if ($first+1==$last) $pager .= '<strong>'.htmlspecialchars($tl['Results:'], ENT_COMPAT, 'ISO-8859-15').'</strong> '.(int)($first+1);
        else                 $pager .= '<strong>'.htmlspecialchars($tl['Results:'], ENT_COMPAT, 'ISO-8859-15').'</strong> '.(int)($first+1).' - '.(int)$last;
    }

    if ($defaultPerPage!=NULL) {
        $pager .= ' | <strong>' . ___('Per page') . ': </strong>';
        $pager .= '<input type="hidden" name="pos" value="0" />';
        $pager .= $GLOBALS['antiCSRF']->getInput('setPerPage');
        $pager .= '<select name="newPerPage" onChange="this.form.submit();">';

        $pager .= '<option value="' . $defaultPerPage . '"' . ($defaultPerPage==$perpage ? ' selected' : '') . '>' . $defaultPerPage . '</option>';
        $pager .= '<option value="' . $defaultPerPage . '"> </option>';
        
        for ($i = 1; $i <= $total; $i = $i*2) {
            if ($i>$perpage && $i/2<$perpage)
                $pager .= '<option value="' . $perpage . '"' . ($perpage==$perpage ? ' selected' : '') . '>' . $perpage . '</option>';
            $pager .= '<option value="' . $i . '"' . ($i==$perpage ? ' selected' : '') . '>' . $i . '</option>';
        }
        $pager .= '<option value="all"' . ($all ? ' selected' : '') . '>' . htmlspecialchars($tl['all'], ENT_COMPAT, 'ISO-8859-15') . '</option>';
        $pager .= '</select>';
    }

    $pager .= '<br />';
    
    if ($first<=0) $pager .= '<span class="passive"><strong>&laquo; '.htmlspecialchars($tl['previous page'], ENT_COMPAT, 'ISO-8859-15').'</strong></span>';
    else           $pager .= '<a href="'.htmlspecialchars($targetURL, ENT_COMPAT, 'ISO-8859-15').'?pos='.(int)($first-$perpage)
                            .'&amp;'.$zusatz_url.'">&laquo; '.htmlspecialchars($tl['previous page'], ENT_COMPAT, 'ISO-8859-15').'</a>';

    $pager .= ' [ ';
    $i = 1;
    while ($i<=$pages) {
        $zaehler = abs(($activePage+1)-$i);
        if ($zaehler<=$GLOBALS['NextSpan'] || $i<=2 || $i>=($pages-1) || (!($i%$nextSpan) && $GLOBALS['NextSpan10'])) {

            if ($i==($activePage+1)) $pager .= '<span class="active"><strong>'.(int)$i.'</strong></span> ';
            else                     $pager .= '<a href="'.htmlspecialchars($targetURL, ENT_COMPAT, 'ISO-8859-15')
                                              .'?pos='.(int)($perpage*($i-1))
                                              .'&amp;'.$zusatz_url.'">'.(int)$i.'</a> ';
            $placeholder_c = false;
            $placeholder_d = false;
       } else {
            if (($activePage+1-$i)>0 && !$placeholder_c) {
                $pager .= $GLOBALS['NextSpanSpace'];
                $placeholder_c = true;
            }

            if (($activePage+1-$i)<0 && !$placeholder_d) {
                $pager .= $GLOBALS['NextSpanSpace'];
                $placeholder_d = true;
            }
        }
        $i++;
    }
    $pager .= '] ';

    if ($last<$total && $perpage<$total) {
        $pager .= '<a href="'.htmlspecialchars($targetURL, ENT_COMPAT, 'ISO-8859-15')
                 .'?pos='.(int)($first+$perpage)
                 .'&amp;'.$zusatz_url.'">'.htmlspecialchars($tl['next page'], ENT_COMPAT, 'ISO-8859-15').' &raquo;</a><br />';
    } else {
        $pager .= '<span class="passive"><strong>'.htmlspecialchars($tl['next page'], ENT_COMPAT, 'ISO-8859-15').' &raquo;</strong></span>';
    }
    if ($defaultPerPage!=NULL) {
        $pager .= '</form>';
    }
    $pager .= '</div>';

    return $pager;
}

function next_page($first, $total, $perpage, $found=false, $zusatz_url='', $targetURL='', $tl=array(), $defaultPerPage = NULL) 
{
    echo string_next_page($first, $total, $perpage, $found, $zusatz_url, $targetURL, $tl, $defaultPerPage);
}

function DEBUG($buglevel,$query,$file,$line) 
{
    global $varDEBUG;
    global $dbPrefix;

    $file = str_replace(CPO_BASEDIR, '', $file);
    
    if (function_exists('fb')) fb($file, $line, $query);

    //$query = htmlspecialchars($query, ENT_COMPAT, 'ISO-8859-15');
    $query = str_replace('WHERE', '<br />WHERE', $query);
    $query = str_replace('FROM', '<br />FROM', $query);
    $query = str_replace('AND', '<br />AND', $query);
    $query = str_replace('OR', '<br />OR', $query);
    $query = str_replace('"', "&quot;", $query);
    $query2 = preg_replace('/<br\s?\/?>/i', ' ', addslashes($query)); 
    
    if ( $varDEBUG != 0 && ( $buglevel <= $varDEBUG || $varDEBUG > 4 ) ) { 
        printf('<span class="debug"><span onclick="prompt(\'Debug: %1$s (%2$d)\', \'%4$s\');">D<span><strong>%1$s&nbsp;<em>(%2$d)</em></strong><br />%3$s</span></span></span>',
            htmlspecialchars($file, ENT_COMPAT, 'ISO-8859-15'),
            htmlspecialchars($line, ENT_COMPAT, 'ISO-8859-15'),
            $query,
            $query2
        );
    }
    return true;
}

function RTECheck() 
{
    $UA = $_SERVER['HTTP_USER_AGENT'];
    $agent     = false;
    $version   = false; 
    $plattform = false;
    switch (true) {
    case (preg_match('/MSIE ([0-9]{1,2}.[0-9]{1,2})/', $UA, $matches)):
        $agent   = 'IE';
        $version = $matches[1];
        break;
    case (preg_match('/Opera ([0-9]{1,2}.[0-9]{1,2})/', $UA, $matches)):
    case (preg_match('/Opera\/([0-9]{1,2}.[0-9]{1,2})/', $UA, $matches)): 
        $agent   = 'OPERA';
        $version = $matches[1];
        break;
    case (preg_match('/Mozilla\/([0-9]{1,2}.[0-9]{1,2})/', $UA, $matches)):
        $agent   = 'MOZ';
        $version = $matches[1];
        break;
    }

    switch (true) {
    case (strstr($UA, 'Win')):
        $plattform = 'Win';
        break;
    case (strstr($UA, 'Linux')):
        $plattform = 'Linux';
        break;
    case (strstr($UA, 'Mac')):
        $plattform = 'Mac';
        break;
    case (strstr($UA, 'Unix')):
        $plattform = 'Unix';
        break;
    }

    switch (true) {
    case ($agent=='MOZ' && version_compare($version, '5.0', '>=')):
        return true;
        break;
    case ($agent=='IE' && $plattform=='Win' && version_compare($version, '5.0', '>=')):
        return true;
        break;
    case ($agent=='OPERA'):
        return true;
        break;
    }
    return false;
}


function help($helpRef, $nowhitespace=false, $show='', $language = '')
{
    global $cpoSystem;

    if (!$cpoSystem['help']) return; 

    $ws = '&nbsp;';
    if ($nowhitespace) $ws = '';

    $alt = __('Sorry, this help is not available.');
    if (!empty($helpRef)) $alt = __('help');

    $img = sprintf('<img src="%1$s" class="help" style="border: 0; vertical-align: text-bottom;" alt="%2$s" title="%2$s">',
                   SKIN_URL.'sym_help.gif', 
                   $alt);

    // no help, sorry
    if (empty($helpRef)) return $ws.$img;

    $link = BASE_REL.'_manual/index.php?ref='.urlencode($helpRef).'&amp;show='.urlencode($show);
    if (!empty($language)) $link .= '&amp;language=' . urlencode($language);

    // return the corresponding link
    $helpstring = 
        $ws.
        '<a href="'.$link.'" target="manual_window" tabindex="32767">'.
        $img.
        '</a>';

    return $helpstring;
}

function update_log_login($log_login_id, $unblock = true)
{
    global $cpoSystem;
    if (empty($log_login_id)) return;

    $db = new DB;
    // remove idle people
    $query = "DELETE FROM ".CPO_LOG_LOGIN." "
            ."WHERE last_action<'".date('Y-m-d H:i:s', time()-$cpoSystem['timeout'])."' ";
    $db->query($query);

    // remove all blocked articles of this user
    if ($unblock) {
        $db->query('DELETE FROM '.CPO_BLOCK.' '
                  .'WHERE log_login_id='.$log_login_id
        );
    }

    // remove blocks on articles where the user is logged out already
    $db->query('SELECT id, log_login_id FROM '.CPO_BLOCK);
    $db2 = new DB;
    while ($db->next_record()) {
        $db2->query('SELECT COUNT(*) FROM '.CPO_LOG_LOGIN.' '.
                    'WHERE id='.$db->v('log_login_id'));
        $db2->next_record();

        if ($db2->v('COUNT(*)')>0) continue;

        $db2->query('DELETE FROM '.CPO_BLOCK.' '.
                    'WHERE id='.$db->v('id'));
    }
}

function user_activity($log_login_id)
{
    global $cpoSystem;

    update_log_login($log_login_id);

    // update last action time of this user
    if (!empty($log_login_id)) {
        $query = "UPDATE ".CPO_LOG_LOGIN." "
                ."SET last_action = '".date('Y-m-d H:i:s', time())."' "
                ."WHERE id = '".addslashes($log_login_id)."'";
        DEBUG(1, $query, __FILE__, __LINE__);
        $db = new DB;
        $db->query($query);
    }
}

function getNavigation() 
{
    $query = 'SELECT id, directory, name, seclevel '
            .'FROM '.CPO_RUBRIC.' ';
    DEBUG(1, $query, __FILE__, __LINE__);
    $db = new DB;
    $db->query($query);
    $rubrics = array(0 => array(
        'id'        => NULL,
        'directory' => NULL,
        'name'      => NULL, 
        'seclevel'  => NULL
    ));
    while ($db->next_record()) {
        $rubrics[$db->v('id')] = array(
            'id'=>$db->v('id'), 
            'directory'=>$db->v('directory'), 
            'name'=>$db->v('name'), 
            'seclevel'=>$db->v('seclevel')
        );
    }

    $query = 'SELECT n.id, n.l, n.r, n.caption, n.rubric_id, n.settings, COUNT(*)-1 AS level, FLOOR((n.r-n.l)/2) AS childs '
            .'FROM '.CPO_NAVTREE.' AS n, '.CPO_NAVTREE.' AS o '
            .'WHERE n.l BETWEEN o.l AND o.r '
            .'GROUP BY n.l '
            .'ORDER BY n.l';
    DEBUG(1, $query, __FILE__, __LINE__);
    $db->query($query);
    $navigation = array();
    while ($db->next_record()) {
        if ($db->v('l')==1) continue;
        initVar($rubrics[$db->v('rubric_id')], NULL); // in case of deleted rubric
        $navigation[] = array(
            'id'=>$db->v('id'),
            'l'=>$db->v('l'),
            'r'=>$db->v('r'),
            'level'=>$db->v('level'),
            'childs'=>$db->v('childs'),
            'caption'=>$db->v('caption'),
            'navigation'=>unserialize($db->v('settings')),
            'rubric'=>$rubrics[$db->v('rubric_id')]
        );
    }

    return $navigation;
}

function license($key, $modid = 0) // 1: comm; 2: ncomm; 3: test; 4: test exp; 5: school
{
    $lic_version = substr($key, 1, 2);
    switch ($lic_version) {
    case '01':
        if ($modid!=0) return false;
        $lic_version = hexdec($lic_version);
        $chk = 0;
        for ($i=0; $i<=29; $i++) { $chk = ($chk+ord($key[$i]))%256; }
        $chksum = substr($key, 30, 2);
        if ($chksum!=sprintf('%02x', $chk)) return false;
        if (hexdec(substr($key, 5, 2).substr($key, 8, 2))<6) return false;
        $typ = hexdec(substr($key, 12, 2));
        if ($typ!=3) return (int)$typ;
        $ts = hexdec(substr($key, 14, 2).substr($key, 18, 2)
                    .substr($key, 22, 2).substr($key, 26, 2));
        if ($ts>=time()) return (int)$typ;
        return ($typ+1);
        break;
    case 'OD':
    case 'od':
    case 'Od':
    case 'oD':
        if (substr($key, 0, 1)!='M' && substr($key, 0, 1)!='m') return false;
        $lic_version = substr($key, 4, 1).substr($key, 7, 1); 
        switch ($lic_version) {
        case '01':
            $lic_version = hexdec($lic_version);
            $chk = 0;
            for ($i=0; $i<=29; $i++) { $chk = ($chk+ord($key[$i]))%256; }
            $chksum = substr($key, 30, 2);
            if ($chksum!=sprintf('%02x', $chk)) return false;
            if (hexdec(substr($key, 5, 2).substr($key, 8, 2))!=$modid) return false;
            $typ = hexdec(substr($key, 12, 2));
            if ($typ!=3) return (int)$typ;
            $ts = hexdec(substr($key, 14, 2).substr($key, 18, 2)
                        .substr($key, 22, 2).substr($key, 26, 2));
            if ($ts>=time()) return (int)$typ;
            return ($typ+1);
            break;
        default:
            return false;
        }
        break;
    default:
        return false;
    }
}

function logaction($action, $comment) 
{
    $login = '';
    if (isset($_SESSION['SID_login']) && $_SESSION['SID_login']!='') 
        $login = formatUserName($_SESSION['SID_user']['id'], $_SESSION['SID_login'], $_SESSION['SID_user']['name']);

    $db = new DB;
    $db->query('DESCRIBE ' . CPO_LOG . ' bemerkung');
    $db->next_record();
    
    $size = 255;
    if (preg_match('/^varchar\(([0-9]*)\)$/i', $db->v('Type'), $matches)==1) {
        $size = $matches[1];
    }
    
    $date = date('Y-m-d H:i:s');
    $comments = str_split($comment, $size);
    $i = 1;
    if (count($comments)==0)  { // Workaround for PHP 8.2 for str_split on empty comment
        $db->query('INSERT INTO '.CPO_LOG.' (user, datum, aktion, bemerkung) '
                  .'VALUES ('
                  ."'" . addslashes($login) . "', "
                  ."'" . $date . "', "
                  ."'" . addslashes($action) . "', "
                  ."'" . addslashes($comment) . "') ");
    } else {
        foreach ($comments as $comment) {
            $db->query('INSERT INTO '.CPO_LOG.' (user, datum, aktion, bemerkung) '
                      .'VALUES ('
                      ."'" . addslashes($login) . "', "
                      ."'" . $date . "', "
                      ."'" . addslashes($action) . (count($comments)>1 ? ' (' . $i++ . '/' . count($comments) . ')': '') . "', "
                      ."'" . addslashes($comment) . "') ");
        }
    }
}

function formatUserName($uid, $login, $name, $email='')
{
    $username = sprintf('%s [%s]', htmlspecialchars($login, ENT_COMPAT, 'ISO-8859-15'), htmlspecialchars($name, ENT_COMPAT, 'ISO-8859-15'));
    if ($email!='') $username .= ' <'.$email.'>';
    return $username;
}

function initVar(&$aVariable, $aValue)
{
	if (!is_array($aValue)) {
		if (!isset($aVariable)) $aVariable = $aValue;
		return;
	}

	if (!isset($aVariable)) $aVariable = array();
	foreach ($aValue as $k => $v) {
        if (!isset($aVariable[$k])) $aVariable[$k] = $v;
    }
	return;
}

function getArticles($rubricId, $pagetype = 'index', $pos = 0, $cpoRubricOverride = array())
{
    initVar($_REQUEST['pos'],NULL);
    
    $globalsBackup = $GLOBALS;
    $posBackup     = $_REQUEST['pos'];

    global $content, $db_autor, $db_changer, $db_content, $db_email,
        $db_idx, $db_initial, $db_last_modify, $db_nr, $db_pub_datum,
        $db_rubric_id, $db_template, $db_titel, $db_verfallsdatum,
        $fetchedcontent, $hdlr_parser_content, $hdlr_parser_replacement,
        $key, $_name, $parameters, $placeholder, $placeholders_cache,
        $replacement, $_type, $typeNames, $cpoSystem, $tl, $activeModules,
        $keepGlobalVars;

    $_REQUEST['pos'] = $pos;

    if(!isset($GLOBALS['cpoRubrics'][$rubricId]))  return "<!-- no rubric $rubricId available -->";

    $cpoRubric = $GLOBALS['cpoRubrics'][$rubricId];
    foreach ($cpoRubricOverride as $k => $v) {
        if (is_array($v)) {
            foreach ($v as $k2 => $v2) $cpoRubric[$k][$k2] = $v2;
            continue;
        }
        $cpoRubric[$k] = $v;
    }
    $cpoRubricSettings = $cpoRubric['settings'];

    ob_start();
    require(CPO_BASEDIR.'_includesite/i_'.$pagetype.'.php');
    $articles = ob_get_clean();

    $_REQUEST['pos'] = $posBackup;

    foreach ($globalsBackup as $gKey => $gValue) {
        if (!in_array($gKey, $keepGlobalVars)) $GLOBALS[$gKey] = $globalsBackup[$gKey]; // FIX for PHP8.1
    }

    return $articles;
}

require_once(CPO_BASEDIR.'_include/mini_title.inc.php');
require_once(CPO_BASEDIR.'_include/mini_status.inc.php');
require_once(CPO_BASEDIR.'_include/mini_publication.inc.php');
require_once(CPO_BASEDIR.'_include/mini_info.inc.php');

// special functions for AK *********************************
function rubricinclude ($rubric_id, $artikelanzahl = 1, $pagetypeuebergabe = 'index', $sortierung = 'sort_id desc') { 
      global $content, $db_autor, $db_changer, $db_content, $db_email, 
          $db_idx, $db_initial, $db_last_modify, $db_nr, $db_pub_datum, 
          $db_rubric_id, $db_template, $db_titel, $db_verfallsdatum, 
          $fetchedcontent, $hdlr_parser_content, $hdlr_parser_replacement, 
          $key, $_name, $parameters, $placeholder, $placeholders_cache, 
          $replacement, $_type, $typeNames, $cpoSystem, $tl, $activeModules, $pagetype; 
    

  switch ($sortierung){ 
     case 'manual': $sortierung = 'sort_id desc'; break; 
     case 'index aufwrts': $sortierung = 'index asc'; break; 
     case 'index abwrts': $sortierung = 'index desc'; break; 
  } 


  ob_start(); 

  // datenbank-abfrage 

  $query = "SELECT rubric_id, commentary, verfallsdatum, templates_id, nr, idx, email, autor, pub_datum, titel, initial, freigabe, changer FROM " . CPO_NEWS . " WHERE " . ( $cpoSystem['release'] ? "artikel_archiv<2 AND freigabe=" . STATE_RELEASED : "artikel_archiv<1 AND freigabe!=" . STATE_BLOCKED ) . " AND pub_datum<='" . date('Y-m-d H:i:s', time()) . "' AND verfallsdatum>='".date('Y-m-d H:i:s', time())."' AND rubric_id='".$rubric_id."' ORDER BY ".$sortierung." LIMIT 0," . $artikelanzahl . " "; 
  $db = new DB; 
  $db->query($query); 

  while($db->next_record()) { 

      $db_template      = $db->v('templates_id');  
      $db_nr            = $db->v('nr'); 
      $db_idx           = $db->v('idx'); 
      $db_email         = $db->v('email'); 
      $db_autor         = $db->v('autor'); 
      $db_changer       = $db->v('changer'); 
      $db_pub_datum     = $db->v('pub_datum'); 
      $db_verfallsdatum = $db->v('verfallsdatum'); 
      $db_commentary    = $db->v('commentary'); 
      $db_titel         = $db->v('titel'); 
      $db_initial       = $db->v('initial'); 
      $db_freigabe      = $db->v('freigabe'); 
      $db_rubric_id     = $db->v('rubric_id'); 
      $oldPagetype = $pagetype; 
      $pagetype = $pagetypeuebergabe; 
      require(CPO_BASEDIR.'_include/select_template.php'); 
      $pagetype = $oldPagetype; 

  } 
  $rubriccontent = ob_get_contents(); 
  ob_end_clean(); 

  return $rubriccontent; 
}
function rubricincludeasc ($rubric_id, $artikelanzahl = 1, $pagetypeuebergabe = 'index', $sortierung = 'titel asc') { 
      global $content, $db_autor, $db_changer, $db_content, $db_email, 
          $db_idx, $db_initial, $db_last_modify, $db_nr, $db_pub_datum, 
          $db_rubric_id, $db_template, $db_titel, $db_verfallsdatum, 
          $fetchedcontent, $hdlr_parser_content, $hdlr_parser_replacement, 
          $key, $_name, $parameters, $placeholder, $placeholders_cache, 
          $replacement, $_type, $typeNames, $cpoSystem, $tl, $activeModules, $pagetype; 
    

  switch ($sortierung){ 
     case 'manual': $sortierung = 'sort_id desc'; break; 
     case 'index aufwrts': $sortierung = 'index asc'; break; 
     case 'index abwrts': $sortierung = 'index desc'; break; 
     case 'titel aufwrts': $sortierung = 'titel asc'; break;
     case 'titel abwrts': $sortierung = 'titel desc'; break;
  } 


  ob_start(); 

  // datenbank-abfrage 

  $query = "SELECT rubric_id, commentary, verfallsdatum, templates_id, nr, idx, email, autor, pub_datum, titel, initial, freigabe, changer FROM " . CPO_NEWS . " WHERE " . ( $cpoSystem['release'] ? "artikel_archiv<2 AND freigabe=" . STATE_RELEASED : "artikel_archiv<1 AND freigabe!=" . STATE_BLOCKED ) . " AND pub_datum<='" . date('Y-m-d H:i:s', time()) . "' AND verfallsdatum>='".date('Y-m-d H:i:s', time())."' AND rubric_id='".$rubric_id."' ORDER BY ".$sortierung." LIMIT 0," . $artikelanzahl . " "; 
  $db = new DB; 
  $db->query($query); 

  while($db->next_record()) { 

      $db_template      = $db->v('templates_id');  
      $db_nr            = $db->v('nr'); 
      $db_idx           = $db->v('idx'); 
      $db_email         = $db->v('email'); 
      $db_autor         = $db->v('autor'); 
      $db_changer       = $db->v('changer'); 
      $db_pub_datum     = $db->v('pub_datum'); 
      $db_verfallsdatum = $db->v('verfallsdatum'); 
      $db_commentary    = $db->v('commentary'); 
      $db_titel         = $db->v('titel'); 
      $db_initial       = $db->v('initial'); 
      $db_freigabe      = $db->v('freigabe'); 
      $db_rubric_id     = $db->v('rubric_id'); 
      $oldPagetype = $pagetype; 
      $pagetype = $pagetypeuebergabe; 
      require(CPO_BASEDIR.'_include/select_template.php'); 
      $pagetype = $oldPagetype; 

  } 
  $rubriccontentasc = ob_get_contents(); 
  ob_end_clean(); 

  return $rubriccontentasc; 
}
// special functions for AK *********************************


// new CPO+ security mechanism
function limitedAccess($minRole)  {
    // the given role is the minimum role to access the page
    if ($_SESSION['SID_user']['admin']<$minRole) {
        logaction('403 Forbidden', 'Unauthorized attempt to access ' . $_SERVER['PHP_SELF'] . "\n" . gethostbyname($_SERVER['REMOTE_ADDR']));
        header('HTTP/1.1 403 Forbidden');
        echo '<html><h1>403 Forbidden</h1><p>You are not supposed to be here.</p></html>';
        exit;
    }
}